// Example1.cpp
//
// SpinCore Technologies, Inc.
// May 2004
// http://www.spincore.com
//
// The following program code uses C API to
// generate and execute a pulse sequence on the PulseBlasterDDS board.

// This program needs to be linked with pbdfuncs.c and os-xxx.c where
// xxx is the operating system you are using. Depending on the opearting
// system, you may also need to link with various driver libraries.

#include "pbdfuncs.h"
#include <stdio.h>

int main(int argc, char **argv)
{
	int start;

	// Locates & Initializes the PulseBlasterDDS Board
	pb_init();

	// Set Because the Board Operates at 100MHz
	set_clock(100);  


	// Prepare the Board to Receive Freqeuncy Values
	start_programming(FREQ_REGS);	

	// Load Frequency Register 0
	set_freq(1.054);
	// Load Frequency Register 1
	set_freq(2);

	
	// Prepare the Board to Receive TX Phase Values
	start_programming(PHASE_REGS_1);

	// Load Phase Register 0
	set_phase(0);
	// Load Phase Register 1
	set_phase(90);


	// Prepare the Board to Receive RX Phase Values
	start_programming(PHASE_REGS_0);

	// Load Phase Register 0
	set_phase(0);
	// Load Phase Register 1
	set_phase(180);
	
	
	// Prepare the Board to Receive pulse program instructions
	start_programming(PULSE_PROGRAM);

	//Instruction 0 - Continue to instruction 1 in 2us
	//Freq Reg 0, Phase Reg 1 for DAC_OUT_1, DDS TX Output ON, Phase Reg 0 for 
	//DAC_OUT_0 and DAC_OUT_2, DDS RX Output ON, Flags = 0x3FF, OPCODE = CONTINUE
	start = pb_inst_2rf(0, 1, TX_ANALOG_ON, 0, RX_ANALOG_ON, 0x3FF, CONTINUE, 0, 2*s);


	// Instruction 1 - Continue to instruction 2 in 4us
	//Freq Reg 0, Phase Reg 1 for DAC_OUT_1, DDS TX Output OFF, Phase Reg 0 for 
	//DAC_OUT_0 and DAC_OUT_2, DDS RX Output ON, Flags = 0x000, OPCODE = CONTINUE
	pb_inst_2rf(0, 1, TX_ANALOG_ON, 0, RX_ANALOG_ON, 0x000, CONTINUE, 0, 4*us);

	// Instruction 2 - Branch to "start" (Instruction 0) in 2us
	//Freq Reg 0, Phase Reg 1 for DAC_OUT_1, DDS TX Output ON, Phase Reg 0 for 
	//DAC_OUT_0 and DAC_OUT_2, DDS RX Output ON, Flags = 0x000, OPCODE = BRANCH
	pb_inst_2rf(0, 1, TX_ANALOG_ON, 0, RX_ANALOG_ON, 0x000, BRANCH, start, 2*us);

	
	// Finished Sending Instructions
	stop_programming();

	// Run the Program
	start_pb();					

	// Release Control of the PulseBlasterDDS Board
	pb_close();

	return 0;
}

