/* SpinCore Technologies, Inc
 * http://www.spincore.com
 * Last modified 2004-08-31
 */

/* OS dependent functions for Linux. Link this file with your 
 * program if you are compiling on linux. Note: if you are using 
 * gcc and you get link errors concerning outb, outl, inb, or inl
 * functions, you need to compile using optimizations. (for 
 * example using flags like -O or -O2)
 */

#include <stdio.h>
#include <sys/io.h>
#include <fcntl.h>
#include "os.h"

static int base_address;

//file descriptor for module
int fd;

/* pb_init ()
 * Initialize the board. This function should be called before 
 * any other board controlling functions are used. If the board 
 * is successfully initiazed, 0 is returned. On error, -1 is
 * returned.
 */
int pb_init ()
{

  
  fd = open("/dev/pbPCI0", O_WRONLY);
  //printf("fd:%d\n", fd);
  if (fd < 0)
    printf("could not get file descriptor for /dev/pbPCI0\n");

  return 0;
}

/* pb_close ()
 * Release control of the board. This should be called when 
 * your control program is done. Returns 0 on success and
 * -1 on failure
 */
int pb_close ()
{
  close(fd);
  return 0;
}

/* write_word ()
 * Write a 4 byte word to the given IO port. Returns 0 on
 * success, -1 otherwise.
 */
int write_word (unsigned int data, unsigned int port)
{
  //printf("writing word\n");
  outl (data, base_address + port);

  return 0;
}

/* write_byte ()
 * Write 1 byte to the given IO port. Returns 0 on success,
 * -1 otherwise.
 */
int write_byte (char data, unsigned int port)
{

  char in[2];
  in[0] = data;
  in[1] = (char)port;
  int i = write(fd, in);

  if (i < 0)
    printf("could not write to device\n");
  return 0;
}

/* read_word ()
 * Read 4 byte word from the given port.
 */
unsigned int read_word (unsigned int port)
{
  return inl (base_address + port);
}

/* read_byte ()
 * Read 1 byte from the given port
 */
char read_byte (unsigned int port)
{
  return inb (base_address + port);
}


